/**
 * Copyright (c) 2008 SunGard
 *     All Rights Reserved
 *        CONFIDENTIAL
 */

package com.netcertainty.common.utils;

import org.apache.commons.codec.binary.Base64;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigInteger;

/**
 * Base64 encode a given value.
 *
 */
public class Base64Decode {

    /**
     * Decode a base64 value.
     * Writes output to standard out.
     * Reports the encoded and decoded data lengths on standard error.
     * 
     * @param args  1. Optional value to decode. If omitted, value is read from standard input.
     */
    public static void main(String[] args)
    {
        // Get options
        int pos = 0;
        int output_radix = 0;
        if (args.length > 0 && ("-r".equals(args[pos]) || "/r".equals(args[pos])))
        {
            pos++;
            if (pos >= args.length) {showUsage(); return;}
            output_radix = Integer.parseInt(args[pos]);
            pos++;
        }
        
        // Get Parameters
        String value;
        if (pos == args.length)
        {
            // Get input from standard in.
            try
            {
                InputStream in = System.in;
                ByteArrayOutputStream bs = new ByteArrayOutputStream();
                int next_byte;
                while ((next_byte = in.read()) != -1)
                {
                    bs.write(next_byte);
                }
                value = bs.toString();
            }
            catch (IOException e)
            {
                System.err.println("Unable to read value to convert.");
                e.printStackTrace();
                return;
            }
        }
        else if (pos + 1 == args.length)
        {
            // Get input from the command line.
            value = args[pos];
        }
        else
        {
            showUsage();
            return;
        }
        value = value.trim();
        
        // Decode the given value
        if (!Base64.isArrayByteBase64(value.getBytes()))
        {
            System.err.println("Data is not valid Base64.");
            return;
        }
        byte[] result = Base64.decodeBase64(value.getBytes());
        
        // Return the result
        System.err.println("Decoded Length: " + result.length);
        System.err.println("Encoded Length: " + value.length());
        if (output_radix <= 1)
        {
            // Output as binary
            for (int i = 0; i < result.length; i++)
            {
                System.out.write(result[i]);
            }
        }
        else
        {
            // Output as big integer in the given radix
            BigInteger bigint = new BigInteger(result);
            System.out.println(bigint.toString(output_radix));
        }
        System.out.flush();
    }
    
    private static void showUsage()
    {
        System.err.println("Usage: Base64Decode [<Options>] [<value>]");
        System.err.println("  Options:");
        System.err.println("    -r <radix> - Set the output radix.");
        System.err.println("        If a radix is provided, outputs a numeric representation");
        System.err.println("        of the decoded value in the given radix.");
        System.err.println("        If a radix is omitted, outputs the decoded value as raw binary.");
        System.err.println("  Parameters:");
        System.err.println("    value - Optional value to decode.");
        System.err.println("            If omitted, value is read from standard input.");
    }
}
