/**
 * Copyright (c) 2008 SunGard
 *     All Rights Reserved
 *        CONFIDENTIAL
 */

package com.netcertainty.common.utils;

import org.apache.commons.codec.binary.Base64;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;

/**
 * Base64 encode a given value.
 *
 */
public class Base64Encode {

    /**
     * Base64 encode a given value.
     * Writes result to standard out.
     * Reports the encoded and decoded data lengths on standard error.
     * 
     * @param args  1. Optional value to encode. If omitted, value is read from standard input.
     */
    public static void main(String[] args)
    {
        // Get Parameters
        byte[] value;
        if (args.length == 0)
        {
            // Get input from standard in.
            int decoded_length = 0;
            try
            {
                InputStream in = System.in;
                ByteArrayOutputStream bs = new ByteArrayOutputStream();
                int next_byte;
                while ((next_byte = in.read()) != -1)
                {
                    bs.write(next_byte);
                    decoded_length++;
                }
                value = bs.toByteArray();
            }
            catch (IOException e)
            {
                System.err.println("Unable to read value to convert. Error at byte # " + decoded_length + ".");
                e.printStackTrace();
                return;
            }
        }
        else if (args.length == 1)
        {
            // Get input from the command line.
            value = args[0].getBytes();
        }
        else
        {
            System.err.println("Usage: Base64Encode [<value>]");
            System.err.println("    value - Optional value to encode.");
            System.err.println("            If omitted, value is read from standard input.");
            return;
        }
        
        // Encode the given value
        String result = new String(Base64.encodeBase64(value));
        
        // Return the results
        System.err.println("Decoded Length: " + value.length);
        System.err.println("Encoded Length: " + result.length());
        System.out.print(result);
    }
}
